#ifndef SETTINGDEF_HPP
#define SETTINGDEF_HPP

#include "EnumeratorSvc.hpp"
#include "MessageSvc.hpp"
#include "StyleSvc.hpp"
#include <vector>
#include "TString.h"

using namespace std;

class EventType;
class FitConfiguration;
class EfficiencyForFitHandler;
/**
 * \namespace SettingDef
 * \brief Setting service
 */
namespace SettingDef {

    extern TString name;

    extern TString debug;

    extern bool verbose;

    extern bool trowLogicError;

    extern bool useMultiThread;

    extern TString separator;

    extern TString blindString;

    /**
     * \class IO
     * \brief IO settings
     */
    class IO {
      public:
        IO() = default;
        static TString exe;
        static TString yaml;

        static TString repoDir;
        static TString anaDir;

        static TString gangaDir;
        static TString ioDir;

        static TString configDir;
        static TString dataDir;

        static TString outDir;

        static bool useEOS;
        static bool useEOSHome;

        static TString eosDir;
        static TString eosHome;
        //ClassDef(IO,1);
    };

    /**
     * \class Config
     * \brief ConfigHolder settings
     */
    class Config {

      public:
        Config() = default;

        static TString project;
        static TString ana;
        static TString sample;

        static TString q2bin;

        static TString year;
        static TString polarity;

        static TString trigger;
        static TString triggerConf;

        static TString brem;
        static TString track;
        //ClassDef(Config,1);

    };

    /**
     * \class Cut
     * \brief CutHolder settings
     */
    class Cut {

      public:
        Cut() = default;
        static TString option;
        static TString extraCut;

        static bool force;
        
        static TString mvaVer;
        //ClassDef(Cut,1);

    };

    /**
     * \class Weight
     * \brief WeightHolder settings
     */
    class Weight {

      public:
        Weight() = default;
        static TString option;
        static TString config;

        static TString trkVer;
        static TString pidVer;
        static TString l0Ver;
        static TString hltVer;
        static TString mcVer;
        static int     iBS;
        static bool    useBS;
        static bool    useStatusL0Formula;
        static bool    priorChain;
        //ClassDef(Weight,1);
    };

    /**
     * \class Tuple
     * \brief TupleHolder settings
     */
    class Tuple {

      public:
        Tuple() = default;
        static TString option;

        static TString gngVer;
        static TString proVer;
        static TString creVer;
        static TString splVer;
        static TString outVer;

        static TString fileName;
        static TString tupleName;

        static TString DT;
        static TString MCT;
        static TString ET;
        static TString LT;
        static TString SPT;
        static TString RST;
        static TString TT;
        static TString TST;

        static bool addTuple;
        static bool dataFrame;
        static bool chainexctrg;

        static bool              branches;
        static vector< TString > branchList;
        static bool              aliases;

        static double frac;

        static bool datasetCache;

        static vector< tuple< TString, vector< tuple< TString, int , double, double, TString ,TString > > > > isoBins;

        static bool noInit; //avoid TupleHolder::Init() to load TChains ( used for BS-fit-loop to fasten the procedure when reloading from Disk )
        //ClassDef(Tuple,1);

    };

    /**
     * \class Events
     * \brief EventTypes settings
     */
    class Events {

      public:
        Events() = default;
        static TString cutOptionCL;
        static TString cutOptionSigMC;
        static TString cutOptionBkgMC;
        static TString weightOption;
        static TString tupleOption;

        static vector< EventType > types;
        static vector< TString >   fails;
        //ClassDef(Events,1);
        
    };

    /**
     * \class Efficiency
     * \brief EfficiencyType settings
     */
    class Efficiency {

      public:
        Efficiency() = default;
        static TString option;

        static TString ver;

        static TString weightOptionSig;
        static TString weightOptionBkg;

        static TString flatnessVer;

        static double minUncertainty;

        static EfficiencyForFitHandler fitconfiguration;
        //ClassDef(Efficiency,1);
    };

    /**
     * \class Fit
     * \brief FitComponent and Fit settings
     */
    class Fit {

      public:
        Fit()=default;
        static TString option;

        static TString ver;

        static bool doBinned;

        static int nBins;

        static TString varSchemeMC;
        static TString varSchemeCL;

        static double stepSize;
        static double stepSizePar;
        static double stepSizeYield;

        static bool splitL0Categories;
        static bool splitTrackCategories;

        static bool plotSumCategories;

        static bool blindYield;
        static bool blindEfficiency;
        static bool blindRatio;

        static bool reduceRooKeysPDF;

        static bool useDatasetCache;
        static bool redoDatasetCache;
        static bool useBremFracCache;

        static TString dumpParamFile;
        static TString initialParamFile;

        static vector< FitConfiguration >                          configurations;
        static map< pair< Prj, Q2Bin >, pair< TString, TString > > yamls;

        static bool useSPlot2;
        //ClassDef(Fit,1);
    };

    /**
     * \class Toy
     * \brief Toy settings
     */
    class Toy {

      public:
        Toy()=default;
        static TString option;

        static TString tupleVer;
        static TString studyVer;

        static int nJobs;
        static int jobIndex;
        static int nToysPerJob;

        static vector< FitConfiguration >                          configurations;
        static map< pair< Prj, Q2Bin >, pair< TString, TString > > yamls;

        static map< TString, map< TString, TString > > TupleConfigGeneratorInputs;
        //ClassDef(Toy,1);
    };

    /**
     * \class AllowedConf
     * \brief Allowed settings
     */
    class AllowedConf {

      public:
        AllowedConf() =  default;
        static const vector< TString >             Projects;
        static const vector< TString >             Analyses;
        static const map< Prj, vector< TString > > Samples;
        static const map< TString, TString >       TexSamples;   // Map for SampleToTex conversion
        static const vector< TString >             Q2Bins;
        static const vector< TString >             Years;
        static const vector< TString >             Polarities;
        static const vector< TString >             L0Categories;
        static const vector< TString >             L0Configurations;
        static const vector< TString >             BremCategories;
        static const vector< TString >             TrackCategories;

        static const vector< TString > CutOptions;
        static const vector< TString > WeightOptions;
        static const vector< TString > WeightConfig;
        static const vector< TString > TupleOptions;
        static const vector< TString > EfficiencyOptions;
        static const vector< TString > FitComponents;
        static const vector< TString > FitOptions;
        //ClassDef(AllowedConf,1);

    };

};   // namespace SettingDef

bool IsBATCH(TString _batch) noexcept;
bool HasEOS() noexcept;

void TrowLogicError(bool _flag) noexcept;
void PrintSettings() noexcept;
void PrintIOSettings() noexcept;
void PrintConfigSettings() noexcept;
void PrintCutSettings() noexcept;
void PrintWeightSettings() noexcept;
void PrintTupleSettings() noexcept;
void PrintEventsSettings() noexcept;
void PrintEfficiencySettings() noexcept;
void PrintFitSettings() noexcept;
void PrintToySettings() noexcept;
void PrintROOTSettings() noexcept;

/**
 * \namespace gRX
 * \brief gRX namespace (similar to gROOT) to retrieve global configurations
 */
namespace gRX {
    TString Project() noexcept;
    TString Analysis() noexcept;
    TString Sample() noexcept;
    TString Q2Bin() noexcept;
    TString Year() noexcept;
    TString Polarity() noexcept;
    TString Trigger() noexcept;
    TString TriggerConf() noexcept;
    TString Brem() noexcept;
    TString Track() noexcept;
};   // namespace gRX

#endif
