#ifndef CUTHOLDERRK_CPP
#define CUTHOLDERRK_CPP

#include "CutHolderRK.hpp"
#include "CutHolderRKst.hpp"

#include "CutDefRK.hpp"
#include "CutDefRX.hpp"
#include "SettingDef.hpp"

#include "TruthMatchingSvc.hpp"

ClassImp(CutHolderRK)

    CutHolderRK::CutHolderRK()
    : m_configHolder() {
    if (SettingDef::debug.Contains("CH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("CutHolderRK", (TString) "Default");
    m_cutOption = SettingDef::Cut::option;
}

CutHolderRK::CutHolderRK(const ConfigHolder & _configHolder, TString _cutOption)
    : m_configHolder(_configHolder) {
    if (SettingDef::debug.Contains("CH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("CutHolderRK", (TString) "ConfigHolder");
    m_cutOption = _cutOption;
}

CutHolderRK::CutHolderRK(const CutHolderRK & _cutHolder)
    : m_configHolder(_cutHolder.GetConfigHolder()) {
    if (SettingDef::debug.Contains("CH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("CutHolderRK", (TString) "CutHolderRK");
    m_cutOption = _cutHolder.Option();
    m_cut       = _cutHolder.Cut();
    m_cuts      = _cutHolder.Cuts();
}

ostream & operator<<(ostream & os, const CutHolderRK & _cutHolder) {
    os << WHITE;
    MessageSvc::Line(os);
    MessageSvc::Print((ostream &) os, "CutHolderRK");
    MessageSvc::Line(os);
    MessageSvc::Print((ostream &) os, "cutOption", _cutHolder.Option());
    if (IsCut(_cutHolder.Cut())) MessageSvc::Print((ostream &) os, "cut", TString(_cutHolder.Cut()));
    MessageSvc::Print((ostream &) os, "cuts", to_string(_cutHolder.Cuts().size()));
    // MessageSvc::Line(os);
    os << RESET;
    return os;
}

void CutHolderRK::Init() {
    MessageSvc::Info(Color::Cyan, "CutHolderRK", (TString) "Initialize ...");
    CreateCut();
    return;
}

void CutHolderRK::CreateCut() {
    if (m_debug) MessageSvc::Debug("CreateCut from CutHolderRKst", m_cutOption);

    m_cut = TCut(NOCUT);
    m_cuts.clear();

    CutHolderRKst _cutHolder = CutHolderRKst(m_configHolder, m_cutOption + "-noPS-noBKG-noPID-noMVA-noQ2-noKSTMASS");
    _cutHolder.Init();

    m_cut  = _cutHolder.Cut();
    m_cuts = _cutHolder.Cuts();

    m_cut = CleanRKstCut(m_cut);
    for (auto _cut : m_cuts) { m_cuts[_cut.first] = CleanRKstCut(_cut.second); }

    // Build new cuts
    if (!m_cutOption.BeginsWith("no-")) {
        if (!m_cutOption.Contains("noPS")) m_cut  = m_cut && GetPreSelectionCut();
        if (!m_cutOption.Contains("noBKG")) m_cut = m_cut && GetBackgroundCut();
        if (!m_cutOption.Contains("noPID")) m_cut = m_cut && GetPIDCut();
        if (!m_cutOption.Contains("noMVA")) m_cut = m_cut && GetMVACut();
        if (!m_cutOption.Contains("noQ2")) m_cut = m_cut && GetQ2Cut();
    }

    m_cut = UpdateDTFCut(m_cut);
    for (auto _cut : m_cuts) { m_cuts[_cut.first] = UpdateDTFCut(_cut.second); }

    if (m_configHolder.IsMC() && m_configHolder.IsSignalMC()) {
        m_cut = UpdateHLT1Cut(m_cut, m_configHolder.GetProject(), m_configHolder.GetAna(), m_configHolder.GetYear(), m_cutOption);
        for (auto _cut : m_cuts) { m_cuts[_cut.first] = UpdateHLT1Cut(_cut.second, m_configHolder.GetProject(), m_configHolder.GetAna(), m_configHolder.GetYear(), m_cutOption); }
    }

    m_cut = ReplaceProject(m_cut, m_configHolder.GetProject());
    for (auto _cut : m_cuts) { m_cuts[_cut.first] = ReplaceProject(_cut.second, m_configHolder.GetProject()); }

    m_cut = CleanCut(m_cut);
    for (auto _cut : m_cuts) { m_cuts[_cut.first] = CleanCut(_cut.second); }

    m_cuts["cutFULL"] = m_cut;

    if (m_debug) MessageSvc::Debug("CreateCut", &m_cut);
    return;
}

TCut CutHolderRK::CleanRKstCut(TCut _cut) {
    if (m_debug) MessageSvc::Debug("CleanRKstCut", &_cut);
    _cut = ReplaceCut(_cut, "B0_", "Bp_");
    _cut = ReplaceCut(_cut, "Pi_", "K_");
    if (m_debug) MessageSvc::Debug("CleanRKstCut", &_cut);
    return _cut;
}

TCut CutHolderRK::GetBackgroundCut() {
    if (m_debug) MessageSvc::Debug("GetBackgroundCut", m_cutOption);

    Analysis ana   = m_configHolder.GetAna();
    Q2Bin    q2bin = m_configHolder.GetQ2bin();
    Year     year  = m_configHolder.GetYear();

    TCut _cut(NOCUT);
    switch (ana) {
        case Analysis::MM: _cut = m_cutOption.Contains("BKGwoPID") ? CutDefRK::Background::peakingBkgMM : CutDefRK::Background::peakingBkgMM_PID; break;
        case Analysis::EE: _cut = m_cutOption.Contains("BKGwoPID") ? CutDefRK::Background::peakingBkgEE : CutDefRK::Background::peakingBkgEE_PID; break;
        default: MessageSvc::Error("GetBackgroundCut", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
    }
    if (m_cutOption.Contains("BKGCOS")) {
        switch (q2bin) {
            case Q2Bin::Low: _cut = _cut && CutDefRK::Background::Bu2DX; break;
            case Q2Bin::Central: _cut = _cut && CutDefRK::Background::Bu2DX; break;
            default: break;
        }
    }
    if (!m_cutOption.Contains("noSL")) {
        switch (ana) {
            case Analysis::MM: _cut = m_cutOption.Contains("BKGwoPID") ? _cut && CutDefRK::Background::semilepBkgMM : _cut && CutDefRK::Background::semilepBkgMM_PID; break;
            case Analysis::EE: _cut = m_cutOption.Contains("BKGwoPID") ? _cut && CutDefRK::Background::semilepBkgEE : _cut && CutDefRK::Background::semilepBkgEE_PID; break;
            default: MessageSvc::Error("GetBackgroundCut", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
        }
    }

    if (!m_cutOption.Contains("BKGwoPID")) {
        _cut = UpdatePIDTune(_cut, to_string(year));
        if (m_cutOption.Contains("PIDMeerkat")) _cut = UpdatePIDMeerkat(_cut, to_string(year));
    }

    if (IsCut(_cut)) m_cuts["cutBKG"] = _cut;

    if (m_debug) MessageSvc::Debug("GetBackgroundCut", &_cut);
    return _cut;
}

TCut CutHolderRK::GetMVACut() {
    if (m_debug) MessageSvc::Debug("GetMVACut", m_cutOption);

    Q2Bin q2bin = m_configHolder.GetQ2bin();

    TCut _cut(NOCUT);
    if (q2bin != Q2Bin::All) {
        Analysis ana  = m_configHolder.GetAna();
        Year     year = m_configHolder.GetYear();
        if (year != Year::All) {
            year = hash_year(GetRunFromYear(to_string(m_configHolder.GetYear())));
        } else {
            year = hash_year(GetRunFromYear(SettingDef::Config::year));
            MessageSvc::Warning("GetMVACut", (TString) "Setting year to", to_string(year));
        }
        //Forcing MVA cut value to be picked from a given Run settings ( Special configuration setup from FitComponent.cpp when -add[2XJPs] sample is used and mixing of years can happen)
        if( m_cutOption.Contains("MVAR1"))   year = Year::Run1;
        if( m_cutOption.Contains("MVAR2p1")) year = Year::Run2p1;
        if( m_cutOption.Contains("MVAR2p2")) year = Year::Run2p2;
        
        auto _config = make_tuple(ana, year, q2bin);

        map< tuple< Analysis, Year, Q2Bin >, TString > _mvaCut = CutDefRK::MVA::CAT;
        // if (m_cutOption.Contains("MVALOOSE") && ((ana == Analysis::EE) && ((q2bin == Q2Bin::Low) || (q2bin == Q2Bin::Central)))) _mvaCut = CutDefRK::MVA::CAT_LOOSE;

        if (_mvaCut.find(_config) == _mvaCut.end()) {
            MessageSvc::Error("Configuration", to_string(get< 0 >(_config)), ",", to_string(get< 1 >(_config)), ",", to_string(get< 2 >(_config)));
            MessageSvc::Error("GetMVACut", (TString) "Invalid configuration (please implement it or use -noMVA)", "EXIT_FAILURE");
        } else {
            _cut = _mvaCut[_config];
            // if (m_cutOption.Contains("noPR") && (ana == Analysis::EE) && (q2bin == Q2Bin::JPsi)) { _cut = TCut(RemoveStringAfter(TString(_cut), " && ")); }
        }
    }

    if (IsCut(_cut)) m_cuts["cutMVA"] = _cut;

    if (m_debug) MessageSvc::Debug("GetMVACut", &_cut);
    return _cut;
}

TCut CutHolderRK::GetPIDCut() {
    if (m_debug) MessageSvc::Debug("GetPIDCut", m_cutOption);

    Analysis ana  = m_configHolder.GetAna();
    Year     year = m_configHolder.GetYear();

    TCut _cut(NOCUT);
    switch (ana) {
        case Analysis::MM: _cut = CutDefRK::PID::pidMM; break;
        case Analysis::EE: _cut = CutDefRK::PID::pidEE; break;
        default: MessageSvc::Error("GetPIDCut", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
    }

    if (m_cutOption.Contains("noProbHH")) _cut = ReplaceCut(_cut, TString(CutDefRK::PID::probnnK), "1");
    if (m_cutOption.Contains("noProbMM")) _cut = ReplaceCut(_cut, TString(CutDefRK::PID::probnnM), "1");
    if (m_cutOption.Contains("noProbEE")) _cut = ReplaceCut(_cut, TString(CutDefRK::PID::probnnE), "1");

    _cut = UpdatePIDTune(_cut, to_string(year));

    if (m_cutOption.Contains("PIDMeerkat")) _cut = UpdatePIDMeerkat(_cut, to_string(year));

    if (IsCut(_cut)) m_cuts["cutPID"] = _cut;

    if (m_debug) MessageSvc::Debug("GetPIDCut", &_cut);
    return _cut;
}

TCut CutHolderRK::GetPreSelectionCut() {
    if (m_debug) MessageSvc::Debug("GetPreSelectionCut", m_cutOption);

    Analysis ana     = m_configHolder.GetAna();
    Year     year    = m_configHolder.GetYear();
    Trigger  trigger = m_configHolder.GetTrigger();

    CutHolderRKst _cutHolder = CutHolderRKst(m_configHolder, m_cutOption);
    _cutHolder.Init();
    TCut _cutMinLPET(NOCUT);
    if (IsCutInMap("cutLPET", _cutHolder.Cuts())) _cutMinLPET = _cutHolder.Cuts().at("cutLPET");
    bool _includeIsMuon = !m_cutOption.Contains("noMuIsMuon");
    TCut _cut(NOCUT);
    switch (ana) {
        case Analysis::MM: _cut = _includeIsMuon? CutDefRK::Quality::qualityMM :  CutDefRK::Quality::qualityMM_noIsMuon ; break;
        case Analysis::EE:
            _cut = CutDefRK::Quality::qualityEE;
            if (!m_cutOption.Contains("noMinLPET")) {
                if (IsCut(_cutMinLPET)) {
                    switch (trigger) {
                        case Trigger::L0I: break;
                        case Trigger::L0L: _cut = _cut && _cutMinLPET; break;
                        case Trigger::All: _cut = _cut && (CutDefRX::Trigger::L0I || _cutMinLPET); break;
                        default: MessageSvc::Error("GetTCKCut", (TString) "Invalid trg", to_string(trigger), "EXIT_FAILURE"); break;
                    }
                }
            }
            break;
        default: MessageSvc::Error("GetPreSelectionCut", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
    }

    if (m_cutOption.Contains("noMinLPET")) {
        if (IsCutInMap("cutLPET", _cutHolder.Cuts())) {
            TCut _cutMinLPET = _cutHolder.Cuts().at("cutLPET");
            if (IsCut(_cutMinLPET)) {
                _cut              = ReplaceCut(_cut, TString(_cutMinLPET), "1");
                m_cuts["cutLPET"] = _cutMinLPET;
            }
        }
    }

    if (IsCut(_cut)) m_cuts["cutPS"] = _cut;

    if (m_debug) MessageSvc::Debug("GetPreSelectionCut", &_cut);
    return _cut;
}

TCut CutHolderRK::GetQ2Cut() {
    if (m_debug) MessageSvc::Debug("GetQ2Cut", m_cutOption);

    Analysis ana   = m_configHolder.GetAna();
    Q2Bin    q2bin = m_configHolder.GetQ2bin();

    TCut _cut(NOCUT);
    switch (q2bin) {
        case Q2Bin::All: break;
        case Q2Bin::Low: _cut = CutDefRK::Mass::Q2Low; break;
        case Q2Bin::Central: _cut = CutDefRK::Mass::Q2Central; break;
        case Q2Bin::High: _cut = ana == Analysis::MM ? CutDefRK::Mass::Q2HighMM : CutDefRK::Mass::Q2HighEE; break;
        case Q2Bin::JPsi: _cut = ana == Analysis::MM ? CutDefRX::Mass::JPsMM : CutDefRX::Mass::JPsEE; break;
        case Q2Bin::Psi: _cut = ana == Analysis::MM ? CutDefRX::Mass::PsiMM : CutDefRX::Mass::PsiEE; break;
        default: MessageSvc::Error("GetQ2Cut", (TString) "Invalid q2bin", to_string(q2bin), "EXIT_FAILURE"); break;
    }
    //Fast cut definer for smeared JPS_M values!
    auto UpdateQ2Cut = []( TCut _cut, TString _cutOption ){
        TString _newCut = TString(_cut); 
        if( _cutOption.Contains("q2SmearMCDTBp") ){
	  _newCut =  _newCut.ReplaceAll("JPs_M", "JPs_M_smear_Bp_fromMCDT_wMC");
        }else if( _cutOption.Contains("q2SmearMCDTB0")){
	  _newCut = _newCut.ReplaceAll("JPs_M", "JPs_M_smear_B0_fromMCDT_wMC");
        }
        else if( _cutOption.Contains("q2SmearBp") ){
	  _newCut =  _newCut.ReplaceAll("JPs_M", "JPs_M_smear_Bp_wMC");
        }else if( _cutOption.Contains("q2SmearB0")){
	  _newCut = _newCut.ReplaceAll("JPs_M", "JPs_M_smear_B0_wMC");
        }
        return TCut( _newCut);
    };
    if( ana == Analysis::EE) _cut = UpdateQ2Cut( _cut, m_cutOption);

    if (IsCut(_cut)) m_cuts["cutQ2"] = _cut;

    if (m_debug) MessageSvc::Debug("GetQ2Cut", &_cut);
    return _cut;
}

#endif
