#ifndef WEIGHTHOLDERRX_CPP
#define WEIGHTHOLDERRX_CPP

#include "WeightHolderRX.hpp"
#include "CutHolder.hpp"

#include "CutDefRX.hpp"
#include "SettingDef.hpp"
#include "WeightDefRX.hpp"

#include "HistogramSvc.hpp"

#include "core.h"
#include "fmt_ostream.h"
#include "vec_extends.h"

#include "TString.h"

ClassImp(WeightHolderRX)

    WeightHolderRX::WeightHolderRX()
    : m_configHolder() {
    if (SettingDef::debug.Contains("WH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("ConfigHolder", (TString) "Default");
    m_weightOption = SettingDef::Weight::option;
    m_weightConfig = SettingDef::Weight::config;
}

WeightHolderRX::WeightHolderRX(const ConfigHolder & _configHolder, TString _weightOption)
    : m_configHolder(_configHolder) {
    if (SettingDef::debug.Contains("WH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("ConfigHolder", (TString) "ConfigHolder");
    m_weightOption = _weightOption;
    m_weightConfig = SettingDef::Weight::config;
}

WeightHolderRX::WeightHolderRX(const WeightHolderRX & _weightHolder)
    : m_configHolder(_weightHolder.GetConfigHolder()) {
    if (SettingDef::debug.Contains("WH")) SetDebug(true);
    if (m_debug) MessageSvc::Debug("WeightHolderRX", (TString) "WeightHolderRX");
    m_weightOption = _weightHolder.Option();
    m_weightConfig = _weightHolder.Config();
    m_weight       = _weightHolder.Weight();
    m_weights      = _weightHolder.Weights();
}

ostream & operator<<(ostream & os, const WeightHolderRX & _weightHolder) {
    os << WHITE;
    MessageSvc::Line(os);
    MessageSvc::Print((ostream &) os, "WeightHolderRX");
    MessageSvc::Line(os);
    MessageSvc::Print((ostream &) os, "weightOption", _weightHolder.Option());
    MessageSvc::Print((ostream &) os, "weightConfig", _weightHolder.Config());
    if (IsWeight(_weightHolder.Weight())) MessageSvc::Print((ostream &) os, "weight", _weightHolder.Weight());
    MessageSvc::Print((ostream &) os, "weights", to_string(_weightHolder.Weights().size()));
    // MessageSvc::Line(os);
    os << RESET;
    return os;
}

void WeightHolderRX::Init() {
    MessageSvc::Info(Color::Cyan, "WeightHolderRX", (TString) "Initialize ...");
    if (!m_weightOption.Contains("MCT") && !m_weightOption.Contains("NORM")) {
      if (! (SettingDef::IO::exe.Contains("upleProcess") || !SettingDef::IO::exe.Contains("submitHLT") )){
            if (m_weightOption.Contains(WeightDefRX::ID::BS) && ((SettingDef::Weight::iBS < 0) || (SettingDef::Weight::iBS > WeightDefRX::nBS))) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            // if (m_weightOption.Contains(WeightDefRX::ID::PID) && !m_weightOption.Contains(WeightDefRX::ID::TRK) && !m_weightConfig.Contains("meerkat")) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            if (m_weightOption.Contains(WeightDefRX::ID::L0) && !m_weightOption.Contains(WeightDefRX::ID::PID) && !m_weightConfig.Contains("meerkat")) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            if (m_weightOption.Contains(WeightDefRX::ID::HLT) && !m_weightOption.Contains(WeightDefRX::ID::L0)) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            // if (m_weightOption.Contains(WeightDefRX::ID::BKIN) && (!m_weightOption.Contains(WeightDefRX::ID::L0) || !m_weightOption.Contains(WeightDefRX::ID::HLT))) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            if (m_weightOption.Contains(WeightDefRX::ID::MULT) && !m_weightOption.Contains(WeightDefRX::ID::BKIN)) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            if (!m_weightOption.Contains(WeightDefRX::ID::BDT)) {
                if ((m_weightOption.Contains(WeightDefRX::ID::RECO) && !m_weightOption.Contains(WeightDefRX::ID::MULT)) && !m_weightOption.Contains(WeightDefRX::ID::PTRECO)) MessageSvc::Error("WeightHolderRX", (TString) "Incompatible Weight options", m_weightOption, "EXIT_FAILURE");
            }
        }
    }
    if (m_weightOption.Contains("TRK") && m_configHolder.GetAna() == Analysis::MM) {
        MessageSvc::Warning("Removing -TRK from WeightOption on Muons, once maps become available, remove this line in WeightHolderRX.cpp");
        m_weightOption = m_weightOption.ReplaceAll("-TRK", "").ReplaceAll("TRK-","").ReplaceAll("TRK", "");
    }
    if (m_configHolder.IsMC())
        CreateWeightMC();
    else
        CreateWeightCL();
    return;
}

void WeightHolderRX::CreateWeightMC() {
    if (m_debug) MessageSvc::Debug("CreateWeightMC", m_weightOption);

    m_weight = TString(NOWEIGHT);

    if (m_weightOption.Contains(WeightDefRX::ID::LB)) {
        if( m_configHolder.GetSample().Contains("Lb2pKJPs") ||  m_configHolder.GetSample().Contains("Lb2pKPsi")){
	        m_weights[WeightDefRX::ID::LB]   = TString(WeightDefRX::LB);
        }
        //m_weight                         = m_weights[WeightDefRX::ID::LB];
        //m_weights[WeightDefRX::ID::FULL] = m_weight;
        //return;
    }

    if (m_weightOption.Contains(WeightDefRX::ID::PTRECO)) {
        m_weights[WeightDefRX::ID::PTRECO] = TString(WeightDefRX::PTRECO);
        //m_weight                           = m_weights[WeightDefRX::ID::PTRECO];
        //m_weights[WeightDefRX::ID::FULL]   = m_weight;
        //return;
    }

    Prj         prj  = m_configHolder.GetProject();
    Analysis    ana  = m_configHolder.GetAna();
    Trigger     trg  = m_configHolder.GetTrigger();
    TriggerConf trgc = m_configHolder.GetTriggerConf();

    if (m_weightOption.Contains(WeightDefRX::ID::BS) && m_configHolder.HasBS() && SettingDef::Weight::useBS && SettingDef::Weight::iBS>=0 && SettingDef::Weight::iBS< WeightDefRX::nBS ) { 
        m_weights[WeightDefRX::ID::BS] = TString(WeightDefRX::BS) + "[" + to_string(SettingDef::Weight::iBS) + "]"; 
    }

    if (m_weightOption.Contains(WeightDefRX::ID::PID)) {
        m_weights[WeightDefRX::ID::PID] = TString(WeightDefRX::PID);
        if( m_weightOption.Contains(WeightDefRX::ID::BS)){
            if( !m_weights[WeightDefRX::ID::PID].Contains("PIDCalib_BS") ){
                m_weights[WeightDefRX::ID::PID].ReplaceAll("PIDCalib", Form("PIDCalib_BS[%i]", SettingDef::Weight::iBS));
            }
        }
    }
    if( ana == Analysis::MM && m_weightOption.Contains(WeightDefRX::ID::ISMUON)){
        m_weights[WeightDefRX::ID::ISMUON] = TString(WeightDefRX::ISMUON); 
    }
    if (m_weightOption.Contains(WeightDefRX::ID::TRK)) { 
        m_weights[WeightDefRX::ID::TRK] = TString(WeightDefRX::TRK); 
        if( m_weightOption.Contains(WeightDefRX::ID::BS)){
            if( !m_weights[WeightDefRX::ID::TRK].Contains("TRKCalib_BS") ){
                m_weights[WeightDefRX::ID::TRK].ReplaceAll("TRKCalib", Form("TRKCalib_BS[%i]", SettingDef::Weight::iBS));
            }
        }
    }

    map< tuple< TriggerConf, Trigger, Analysis >, TString > _l0Weight =  WeightDefRX::wL0 ;
    if( m_weightOption.Contains("COMB")){
        MessageSvc::Warning("Using dilepton weight");
        _l0Weight = WeightDefRX::wL0comb ;
    }
    if (m_weightOption.Contains(WeightDefRX::ID::L0)) {         
        m_weights[WeightDefRX::ID::L0] = TString(_l0Weight.at(make_tuple(trgc, trg, ana))); 
        if( m_weightOption.Contains(WeightDefRX::ID::BS)){
            if( !m_weights[WeightDefRX::ID::L0].Contains("effCL_BS") ){
                m_weights[WeightDefRX::ID::L0].ReplaceAll("effCL", Form("effCL_BS[%i]", SettingDef::Weight::iBS));
            }
            if( !m_weights[WeightDefRX::ID::L0].Contains("effMC_BS")){
                m_weights[WeightDefRX::ID::L0].ReplaceAll("effMC", Form("effMC_BS[%i]",  SettingDef::Weight::iBS));
            }
        }
    }

    map< pair< TriggerConf, Trigger >, TString > _hltWeight = WeightDefRX::wHLT;
    if (m_weightOption.Contains(WeightDefRX::ID::HLT)) { 
        m_weights[WeightDefRX::ID::HLT] = TString(_hltWeight.at(make_pair(trgc, trg))); 
        if( m_weightOption.Contains(WeightDefRX::ID::BS)){
            if( !m_weights[WeightDefRX::ID::HLT].Contains("effCL_BS") ){
                m_weights[WeightDefRX::ID::HLT].ReplaceAll("effCL", Form("effCL_BS[%i]", SettingDef::Weight::iBS));
            }
            if( !m_weights[WeightDefRX::ID::HLT].Contains("effMC_BS")){
                m_weights[WeightDefRX::ID::HLT].ReplaceAll("effMC", Form("effMC_BS[%i]",  SettingDef::Weight::iBS));
            }
        }
    }

    if (m_weightOption.Contains(WeightDefRX::ID::BDT)) {
        if( m_weightOption.Contains("fromLOI")){
	        m_weights[WeightDefRX::ID::BDT] = TString(WeightDefRX::BDT_L0I);
	    }else{
	        m_weights[WeightDefRX::ID::BDT] = TString(WeightDefRX::BDT);
            if( m_weightOption.Contains(WeightDefRX::ID::BS)){
                if( !m_weights[WeightDefRX::ID::BDT].Contains("BDT_BS_{OPT}_{B}_MM_L0L") ){
                    m_weights[WeightDefRX::ID::BDT].ReplaceAll("BDT_{OPT}_{B}_MM_L0L", Form("BDT_BS_{OPT}_{B}_MM_L0L[%i]", SettingDef::Weight::iBS));
                }
            }
	    }
        if (!m_weightOption.Contains("MCT") && m_weightOption.Contains(WeightDefRX::ID::RECO)){
            m_weights[WeightDefRX::ID::BDT2] = TString(WeightDefRX::BDT2);
        }
    } else {
        if (m_weightOption.Contains(WeightDefRX::ID::BKIN)) { 
            m_weights[WeightDefRX::ID::BKIN] = TString(WeightDefRX::BKIN); 
        }
        if (m_weightOption.Contains(WeightDefRX::ID::MULT)) { 
            m_weights[WeightDefRX::ID::MULT] = TString(WeightDefRX::MULT); 
        }
        if (!m_weightOption.Contains("MCT")) {
            if ((m_weightOption.Contains(WeightDefRX::ID::RECO) && !m_weightOption.Contains(WeightDefRX::ID::PTRECO))) { m_weights[WeightDefRX::ID::RECO] = TString(WeightDefRX::RECO); }
        }
    }

    // CutHolder _cutHolder = CutHolder(m_configHolder, "");
    // if (m_weightOption.Contains(WeightDefRX::ID::HLT)) _cutHolder.Init();

    if (trg == Trigger::All) {
        if (m_debug) MessageSvc::Debug("Trigger::All", m_weightOption);
        // BUILDS A || (B && !A)
        if (m_weightOption.Contains(WeightDefRX::ID::L0)) {
            MessageSvc::Error("CreateWeightMC", (TString) "Trigger::All WeightDefRX::ID::L0 not implemented");
            /*
            TString _weight;
            _weight += "((";
            // BUILDS A
            switch (trgc) {
                case TriggerConf::Exclusive: _weight += TString(WeightDefRX::L0I_inc); break;
                case TriggerConf::Exclusive2:
                    switch (ana) {
                        case Analysis::MM: _weight += TString(WeightDefRX::L0M_inc); break;
                        case Analysis::EE: _weight += TString(WeightDefRX::L0E_inc); break;
                        default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
                    }
                    break;
                default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
            }
            if (m_weightOption.Contains(WeightDefRX::ID::HLT)) {
                switch (trgc) {
                    case TriggerConf::Exclusive: _weight += " * " + ((TString) WeightDefRX::HLT_inc).ReplaceAll("_L0_", "_L0I_"); break;
                    case TriggerConf::Exclusive2: _weight += " * " + ((TString) WeightDefRX::HLT_inc).ReplaceAll("_L0_", "_L0L_"); break;
                    default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
                }
                _weight += " * (" + TString(_cutHolder.Cut("cutHLT1") && _cutHolder.Cut("cutHLT2")) + ")";
            }
            if (m_weightOption.Contains(WeightDefRX::ID::BDT)) {
                _weight += " * " + TString(WeightDefRX::BDT);
                if (!m_weightOption.Contains("MCT") && m_weightOption.Contains(WeightDefRX::ID::BDT2) && ana == Analysis::EE && m_weightOption.Contains(WeightDefRX::ID::RECO)) _weight += " * " + TString(WeightDefRX::BDT2);
            } else {
                if (m_weightOption.Contains(WeightDefRX::ID::BKIN)) { _weight += " * " + TString(WeightDefRX::BKIN); }
                if (m_weightOption.Contains(WeightDefRX::ID::MULT)) { _weight += " * " + TString(WeightDefRX::MULT); }
                if ((m_weightOption.Contains(WeightDefRX::ID::RECO) && !m_weightOption.Contains(WeightDefRX::ID::PTRECO))) { _weight += " * " + TString(WeightDefRX::RECO); }
            }
            if (m_weightConfig.Contains("MML0Lincl")) {
                _weight.ReplaceAll("wBKIN_{B}_{LL}_{L0}", "wBKIN_{B}_MM_L0L_incl");
                _weight.ReplaceAll("wMULT_{B}_{LL}_{L0}", "wMULT_{B}_MM_L0L_incl");
                _weight.ReplaceAll("_{pre}", "_MML0Lincl");
            } else {
                _weight.ReplaceAll("_{pre}", "");
            }
            switch (trgc) {
                case TriggerConf::Exclusive: _weight.ReplaceAll("{L0}", to_string(Trigger::L0I) + "_incl"); break;
                case TriggerConf::Exclusive2: _weight.ReplaceAll("{L0}", to_string(Trigger::L0L) + "_incl"); break;
                default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
            }

            _weight += ") + (";

            // BUILDS (B && !A)
            switch (trgc) {
                case TriggerConf::Exclusive:
                    switch (ana) {
                        case Analysis::MM: _weight += TString(WeightDefRX::L0M_exc); break;
                        case Analysis::EE: _weight += TString(WeightDefRX::L0E_exc); break;
                        default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
                    }
                    _weight += " * (1 - " + TString(CutDefRX::Trigger::L0I) + ")";
                    break;
                case TriggerConf::Exclusive2:
                    _weight += TString(WeightDefRX::L0I_exc);
                    switch (ana) {
                        case Analysis::MM: _weight += " * (1 - (" + TString(CutDefRX::Trigger::L0M) + "))"; break;
                        case Analysis::EE: _weight += " * (1 - (" + TString(CutDefRX::Trigger::L0E) + "))"; break;
                        default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid ana", to_string(ana), "EXIT_FAILURE"); break;
                    }
                    break;
                default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
            }
            if (m_weightOption.Contains(WeightDefRX::ID::HLT)) {
                switch (trgc) {
                    case TriggerConf::Exclusive: _weight += " * " + ((TString) WeightDefRX::HLT_exc).ReplaceAll("_L0_", "_L0L_"); break;
                    case TriggerConf::Exclusive2: _weight += " * " + ((TString) WeightDefRX::HLT_exc).ReplaceAll("_L0_", "_L0I_"); break;
                    default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
                }
                _weight += " * (" + TString(_cutHolder.Cut("cutHLT1") && _cutHolder.Cut("cutHLT2")) + ")";
            }
            if (m_weightOption.Contains(WeightDefRX::ID::BDT)) {
                _weight += " * " + TString(WeightDefRX::BDT);
                if (!m_weightOption.Contains("MCT") && m_weightOption.Contains(WeightDefRX::ID::BDT2) && ana == Analysis::EE && m_weightOption.Contains(WeightDefRX::ID::RECO)) _weight += " * " + TString(WeightDefRX::BDT2);
            } else {
                if (m_weightOption.Contains(WeightDefRX::ID::BKIN)) { _weight += " * " + TString(WeightDefRX::BKIN); }
                if (m_weightOption.Contains(WeightDefRX::ID::MULT)) { _weight += " * " + TString(WeightDefRX::MULT); }
                if ((m_weightOption.Contains(WeightDefRX::ID::RECO) && !m_weightOption.Contains(WeightDefRX::ID::PTRECO))) { _weight += " * " + TString(WeightDefRX::RECO); }
            }
            if (m_weightConfig.Contains("MML0Lincl")) {
                _weight.ReplaceAll("wBKIN_{B}_{LL}_{L0}", "wBKIN_{B}_MM_L0L_incl");
                _weight.ReplaceAll("wMULT_{B}_{LL}_{L0}", "wMULT_{B}_MM_L0L_incl");
                _weight.ReplaceAll("_{pre}", "_MML0Lincl");
            } else {
                _weight.ReplaceAll("_{pre}", "");
            }
            switch (trgc) {
                case TriggerConf::Exclusive: _weight.ReplaceAll("{L0}", to_string(Trigger::L0L) + "_excl"); break;
                case TriggerConf::Exclusive2: _weight.ReplaceAll("{L0}", to_string(Trigger::L0I) + "_excl"); break;
                default: MessageSvc::Error("CreateWeightMC", (TString) "Invalid trgc", to_string(trgc), "EXIT_FAILURE"); break;
            }

            _weight += "))";

            m_weights[WeightDefRX::ID::TRGALL] = _weight;
            */
        }
    }

    for (auto _weight : m_weights) {
        if (_weight.second == "") MessageSvc::Error("CreateWeightMC", (TString) "Invalid", _weight.first, "weight", "EXIT_FAILURE");
    }

    for (auto _weight : m_weights) { m_weights[_weight.first] = UpdateNamesWeightMC(_weight.first, _weight.second); }
    for (auto _weight : m_weights) { m_weights[_weight.first] = TString(ReplaceProject(TCut(_weight.second), m_configHolder.GetProject())); }
    for (auto _weight : m_weights) { m_weights[_weight.first] = CleanWeight(_weight.second); }

    if (IsWeightInMap(WeightDefRX::ID::BS, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::BS];
    if (IsWeightInMap(WeightDefRX::ID::PID, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::PID];
    if (IsWeightInMap(WeightDefRX::ID::ISMUON, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::ISMUON];    
    if (IsWeightInMap(WeightDefRX::ID::TRK, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::TRK];
    if (trg == Trigger::All) {
        if (IsWeightInMap(WeightDefRX::ID::TRGALL, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::TRGALL];
    } else {
        if (IsWeightInMap(WeightDefRX::ID::L0, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::L0];
    }
    if (IsWeightInMap(WeightDefRX::ID::HLT, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::HLT];
    if (IsWeightInMap(WeightDefRX::ID::BDT, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::BDT];
    if (IsWeightInMap(WeightDefRX::ID::BDT2, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::BDT2];

    if (IsWeightInMap(WeightDefRX::ID::LB, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::LB];
    if (IsWeightInMap(WeightDefRX::ID::PTRECO, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::PTRECO];

    m_weight = CleanWeight(m_weight);
    if (IsWeight(m_weight)) m_weight = "(" + m_weight + ")";

    m_weights[WeightDefRX::ID::FULL] = m_weight;

    if (m_debug) MessageSvc::Debug("CreateWeightMC", m_weight);
    return;
}

TString WeightHolderRX::UpdateNamesWeightMC(TString _name, TString _weight) {
    if (m_debug) MessageSvc::Debug("UpdateNamesWeightMC", _name, _weight);

    Prj      prj = m_configHolder.GetProject();
    Analysis ana = m_configHolder.GetAna();
    Trigger  trg = m_configHolder.GetTrigger();

    if (m_weightOption.Contains(WeightDefRX::ID::TRK) && _name.Contains(WeightDefRX::ID::TRK)) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);

        if (m_weightConfig.Contains("kde"))
            // _weight.ReplaceAll("w", "wi"); 
            _weight.ReplaceAll("w", "w"); //use no interpolation for TRK weights (since Marteen update on 3D maps (Phi,Eta,PT based))
        else if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("meerkat"))
            // _weight.ReplaceAll("w", "wi");
            _weight.ReplaceAll("w", "w"); //use no interpolation for TRK weights (since Marteen update on 3D maps (Phi,Eta,PT based))
        else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");
    }

    if (m_weightOption.Contains(WeightDefRX::ID::PID) && _name.Contains(WeightDefRX::ID::PID)) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);

        if (m_weightConfig.Contains("kde"))
            //_weight.ReplaceAll("w", "wk");
            _weight.ReplaceAll("PIDCalib", "PIDCalibKDE");
        else if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde")) {
            _weight.ReplaceAll("w", "wi");
        } else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");
    }
    if(m_weightOption.Contains(WeightDefRX::ID::ISMUON) && _name.Contains(WeightDefRX::ID::ISMUON)) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);
        if (m_weightConfig.Contains("kde"))
            _weight.ReplaceAll("PIDCalib", "PIDCalibKDE");
        else if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde") || m_weightConfig.Contains("meerkat")) {
            _weight.ReplaceAll("w", "wi");
        } else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");
    }
    if ((m_weightOption.Contains(WeightDefRX::ID::BDT) || m_weightOption.Contains(WeightDefRX::ID::BDT2)) && (_name.Contains(WeightDefRX::ID::BDT) || _name.Contains(WeightDefRX::ID::BDT2) || _name.Contains(WeightDefRX::ID::TRGALL))) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);
        if (_name.Contains(WeightDefRX::ID::BDT2) && ana == Analysis::EE) {
            _weight.ReplaceAll("BDT2", "BDT3");
            _weight.ReplaceAll("{L0}", to_string(trg));
        }
        if (_name.Contains(WeightDefRX::ID::BDT2) && ana == Analysis::MM) _weight.ReplaceAll("{L0}", "L0L");
        if (m_weightOption.Contains(WeightDefRX::ID::BKIN) && m_weightOption.Contains(WeightDefRX::ID::MULT))
            _weight.ReplaceAll("{OPT}", "BKIN_MULT");
        else if (m_weightOption.Contains(WeightDefRX::ID::BKIN))
            _weight.ReplaceAll("{OPT}", "BKIN");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightOption", m_weightOption, "EXIT_FAILURE");
        if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde"))
            _weight.ReplaceAll("w", "wf");   // these branches are based on fitted L0 weights
        else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");   // these branches are based on no interpolated binned L0 weights
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");   // these branches are based on interpolated binned L0 weights
        else if (m_weightConfig.Contains("meerkat"))
            _weight.ReplaceAll("w", "wf");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        if (m_weightConfig.Contains("Bp"))
            _weight.ReplaceAll("{B}", "Bp");
        else if (m_weightConfig.Contains("B0"))
            _weight.ReplaceAll("{B}", "B0");
        else if (m_weightConfig.Contains("Bs"))
            _weight.ReplaceAll("{B}", "Bs");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        _weight.ReplaceAll("{LL}", to_string(m_configHolder.GetAna()));

        _weight.ReplaceAll("{L0}", m_configHolder.GetTriggerAndConf("short"));

    } else if ((m_weightOption.Contains(WeightDefRX::ID::BKIN) || m_weightOption.Contains(WeightDefRX::ID::MULT) || m_weightOption.Contains(WeightDefRX::ID::RECO)) && (_name.Contains(WeightDefRX::ID::BKIN) || _name.Contains(WeightDefRX::ID::MULT) || _name.Contains(WeightDefRX::ID::RECO) || _name.Contains(WeightDefRX::ID::TRGALL))) {

        if (m_debug) MessageSvc::Debug("Weight " + _name);

        if (m_weightConfig.Contains("MML0Lincl")) {
            _weight.ReplaceAll("{LL}_{L0}", "MM_L0L_incl");
            _weight.ReplaceAll("_{pre}", "_MML0Lincl");
        } else if (m_weightConfig.Contains("MML0Lexcl")) {
            _weight.ReplaceAll("{LL}_{L0}", "MM_L0L_excl");
            _weight.ReplaceAll("_{pre}", "_MML0Lexcl");
        } else
            _weight.ReplaceAll("_{pre}", "");

        if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde"))
            _weight.ReplaceAll("w", "wf");   // these branches are based on fitted L0 weights
        else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");   // these branches are based on no interpolated binned L0 weights
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");   // these branches are based on interpolated binned L0 weights
        else if (m_weightConfig.Contains("meerkat"))
            _weight.ReplaceAll("w", "wf");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        if (m_weightConfig.Contains("Bp"))
            _weight.ReplaceAll("{B}", "Bp");
        else if (m_weightConfig.Contains("B0"))
            _weight.ReplaceAll("{B}", "B0");
        else if (m_weightConfig.Contains("Bs"))
            _weight.ReplaceAll("{B}", "Bs");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        _weight.ReplaceAll("{LL}", to_string(m_configHolder.GetAna()));
        // if ((m_configHolder.GetAna() == Analysis::EE) && m_weightConfig.Contains("EE")) _weight.ReplaceAll("MM", "EE");

        _weight.ReplaceAll("{L0}", m_configHolder.GetTriggerAndConf("short"));

        // if (m_weightConfig.Contains("L0I"))
        //    _weight.ReplaceAll("{L0}", "L0I");
        // else if (m_weightConfig.Contains("L0L"))
        //    _weight.ReplaceAll("{L0}", "L0L");
        // else
        //    MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        // if (m_weightConfig.Contains("inclusive"))
        //    _weight.ReplaceAll("", "");
        // else if (m_weightConfig.Contains("exclusive2"))
        //    _weight.ReplaceAll("L0I", "L0I_exclusive");
        // else if (m_weightConfig.Contains("exclusive"))
        //    _weight.ReplaceAll("L0L", "L0L_exclusive");
        // else
        //    MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");
    }

    if (m_weightOption.Contains(WeightDefRX::ID::L0) && (_name.Contains(WeightDefRX::ID::L0) || _name.Contains(WeightDefRX::ID::TRGALL))) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);

        if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde"))
            _weight.ReplaceAll("w", "wf");
        else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");
        else if (m_weightConfig.Contains("meerkat"))
            _weight.ReplaceAll("w", "wf");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        if (m_weightConfig.Contains("Bp"))
            _weight.ReplaceAll("{B}", "Bp");
        else if (m_weightConfig.Contains("B0"))
            _weight.ReplaceAll("{B}", "B0");
        else if (m_weightConfig.Contains("Bs"))
            _weight.ReplaceAll("{B}", "Bs");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        vector< pair< TString, TString > > _heads = GetParticleBranchNames(m_configHolder.GetProject(), m_configHolder.GetAna(), m_configHolder.GetQ2bin(), "onlyhead");
        _weight.ReplaceAll("{B}", _heads[0].first).ReplaceAll("{HEAD}", _heads[0].first);

        //Important bit 
        if(SettingDef::Weight::useStatusL0Formula == false){
            if (m_configHolder.GetYear() == Year::Run1) {
                _weight.ReplaceAll("E1_L0ElectronDecision_TOS",Form("((E1_L0Calo_ECAL_realET > %f && Year == 11) || (E1_L0Calo_ECAL_realET > %f && Year == 12))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2011), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2012)));
                _weight.ReplaceAll("E2_L0ElectronDecision_TOS",Form("((E2_L0Calo_ECAL_realET > %f && Year == 11) || (E2_L0Calo_ECAL_realET > %f && Year == 12))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2011), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2012)));
            }
            else if (m_configHolder.GetYear() == Year::Run2p1) {
                _weight.ReplaceAll("E1_L0ElectronDecision_TOS",Form("((E1_L0Calo_ECAL_realET > %f && Year == 15) || (E1_L0Calo_ECAL_realET > %f && Year == 16))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2015), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2016)));
                _weight.ReplaceAll("E2_L0ElectronDecision_TOS",Form("((E2_L0Calo_ECAL_realET > %f && Year == 15) || (E2_L0Calo_ECAL_realET > %f && Year == 16))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2015), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2016)));
            }
            else if (m_configHolder.GetYear() == Year::Run2p2) {
                _weight.ReplaceAll("E1_L0ElectronDecision_TOS",Form("((E1_L0Calo_ECAL_realET > %f && Year == 17) || (E1_L0Calo_ECAL_realET > %f && Year == 18))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2017), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2018)));
                _weight.ReplaceAll("E2_L0ElectronDecision_TOS",Form("((E2_L0Calo_ECAL_realET > %f && Year == 17) || (E2_L0Calo_ECAL_realET > %f && Year == 18))", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2017), CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( Year::Y2018)));
            } else {
                _weight.ReplaceAll("E1_L0ElectronDecision_TOS",Form("(E1_L0Calo_ECAL_realET > %f)", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( m_configHolder.GetYear())));
                _weight.ReplaceAll("E2_L0ElectronDecision_TOS",Form("(E2_L0Calo_ECAL_realET > %f)", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( m_configHolder.GetYear())));                
            }
            _weight.ReplaceAll("M1_L0MuonDecision_TOS","(1>0)");
            _weight.ReplaceAll("M2_L0MuonDecision_TOS","(1>0)");            
        }else{
            _weight.ReplaceAll("E1_L0ElectronDecision_TOS",Form("(E1_L0ElectronDecision_TOS && E1_L0Calo_ECAL_realET > %f)", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( m_configHolder.GetYear())));
            _weight.ReplaceAll("E2_L0ElectronDecision_TOS",Form("(E2_L0ElectronDecision_TOS && E2_L0Calo_ECAL_realET > %f)", CutDefRX::Quality::L0Calo_ECAL_realET_Thresholds.at( m_configHolder.GetYear())));
        }
        //The final L0 weight will be as baseline ( useStatusL0Formula == false by default )
        //P(data) = ( 1- ( 1- w1*( P1_realET))*(1- w2*( P2_realET )))
        //For useStatusL0Formula =True : 
        //P(data) = ( 1- ( 1- w1*( P1_TOS && realET))*(1- w2*(P2_TOS && realET )))
    }

    if (m_weightOption.Contains(WeightDefRX::ID::HLT) && (_name.Contains(WeightDefRX::ID::HLT) || _name.Contains(WeightDefRX::ID::TRGALL))) {
        if (m_debug) MessageSvc::Debug("Weight " + _name);

        if (m_weightConfig.Contains("fit") || m_weightConfig.Contains("kde"))
            _weight.ReplaceAll("w", "wi");   // Bp_fit chained to w (interp)
        else if (m_weightConfig.Contains("nointerp"))
            _weight.ReplaceAll("wi", "w");
        else if (m_weightConfig.Contains("interp"))
            _weight.ReplaceAll("w", "wi");
        else if (m_weightConfig.Contains("meerkat"))
            _weight.ReplaceAll("w", "wi");   // meerkat chaining with interp
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");

        if (trg != Trigger::All) {
            switch (trg) {
                case Trigger::L0I: _weight.ReplaceAll("_{L0}_", "_L0I_"); break;
                case Trigger::L0L: _weight.ReplaceAll("_{L0}_", "_L0L_"); break;
                default: MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid trg", to_string(trg), "EXIT_FAILURE"); break;
            }
        }

        if (m_weightConfig.Contains("Bp"))
            _weight.ReplaceAll("{B}", "Bp");
        else if (m_weightConfig.Contains("B0"))
            _weight.ReplaceAll("{B}", "B0");
        else if (m_weightConfig.Contains("Bs"))
            _weight.ReplaceAll("{B}", "Bs");
        else
            MessageSvc::Error("UpdateNamesWeightMC " + _name, (TString) "Invalid weightConfig", m_weightConfig, "EXIT_FAILURE");
        if( m_weightOption.Contains( WeightDefRX::ID::HLT +"-nTracks")){
            _weight.ReplaceAll("HLT_","HLTnTracks_");
        }
        if( m_weightOption.Contains( WeightDefRX::ID::HLT +"-BETA")){
            _weight.ReplaceAll("HLT_","HLTBETA_");
        }
    }

    if (m_debug) MessageSvc::Debug("UpdateNamesWeightMC", _weight);
    return _weight;
}

void WeightHolderRX::CreateWeightCL() {
    if (m_debug) MessageSvc::Debug("CreateWeightCL", m_weightOption);

    m_weight = TString(NOWEIGHT);

    if (m_weightOption.Contains(WeightDefRX::ID::SP)) {
        Q2Bin q2bin = m_configHolder.GetQ2bin();

        TString _weight = "nsig_";
        switch (q2bin) {
            case Q2Bin::JPsi: _weight += "JPs"; break;
            default: MessageSvc::Error("CreateWeightCL", (TString) "Invalid q2bin", to_string(q2bin), "EXIT_FAILURE"); break;
        }
        _weight += SettingDef::separator;
        _weight += m_configHolder.GetKey();
        _weight += "_sw";
        _weight.ReplaceAll("-", "M");   // WEIRD FEATURE OF GetClonedTree
        _weight = CleanWeight(_weight);

        m_weights[WeightDefRX::ID::SP] = _weight;
    }

    if (m_weightOption.Contains(WeightDefRX::ID::BS) && m_configHolder.HasBS() && SettingDef::Weight::useBS && SettingDef::Weight::iBS>=0 && SettingDef::Weight::iBS< WeightDefRX::nBS){ 
        m_weights[WeightDefRX::ID::BS] = TString(WeightDefRX::BS) + "[" + to_string(SettingDef::Weight::iBS) + "]"; 
    }
    
    if (IsWeightInMap(WeightDefRX::ID::BS, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::BS];
    if (IsWeightInMap(WeightDefRX::ID::SP, m_weights)) m_weight += " * " + m_weights[WeightDefRX::ID::SP];

    m_weight = CleanWeight(m_weight);
    m_weight = "(" + m_weight + ")";

    m_weights[WeightDefRX::ID::FULL] = m_weight;

    if (m_debug) MessageSvc::Debug("CreateWeightCL", m_weight);
    return;
}

#endif
