import ROOT as r
class FlatnessParEvaluator : 
    def __init__(self, inFile, varID, wConfig, year, trg, trgConf,  wOpt, sysQ2 = "psi" ):
        self.inFile = r.TFile(inFile)
        self.varID  = varID 
        print( self.varID)        
        self.wConf  = wConfig
        self.year   = year
        self.trg    = trg
        self.trgConf = trgConf
        self.wOpt    = wOpt 
        self.sysQ2   = sysQ2
        self.extractHistos() 
    def extractHistos(self) : 
        to_get_path = "{vID}/{wConf}".format( vID= self.varID, wConf = self.wConf)
        keyBind     = "{wConf}_chain_{year}_{trg}_{trgConf}_{wOpt}".format( wConf = self.wConf,
                                                                            year = self.year,
                                                                            trg = self.trg,
                                                                            trgConf = self.trgConf,
                                                                            wOpt = self.wOpt )
        #print( keyBind)
        #print( self.varID)
        self.histos = { 
            "effEE_jps"     : self.inFile.Get( "{}/{}_EE_eff_jps".format( to_get_path, keyBind )),
            "effMM_jps"     : self.inFile.Get( "{}/{}_MM_eff_jps".format( to_get_path, keyBind )),
            "effEE_otherQ2" : self.inFile.Get( "{}/{}_EE_eff_{}".format( to_get_path, keyBind , self.sysQ2)),
            "effMM_otherQ2" : self.inFile.Get( "{}/{}_MM_eff_{}".format( to_get_path, keyBind , self.sysQ2)),
            "rJpsi"         : self.inFile.Get( "{}/{}".format( to_get_path, keyBind)),
            "Yields_EE"     : self.inFile.Get( "{}/{}_EE_Yields_jps".format( to_get_path, keyBind)),
            "Yields_MM"     : self.inFile.Get( "{}/{}_MM_Yields_jps".format( to_get_path, keyBind)),		
            "Yields_CorrEE" : self.inFile.Get( "{}/{}_EE_effCorr".format( to_get_path, keyBind)),
            "Yields_CorrMM" : self.inFile.Get( "{}/{}_MM_effCorr".format( to_get_path, keyBind))
        }
        for h in self.histos.keys() : 
            #print(h)
            self.histos[h].SetDirectory(0)
        self.evalFlatnessSyst()
    def evalFlatnessSyst(self): 
        """
        TH1D::operaror*
           TH1C hnew = h1;
           hnew.Multiply(&h2);
           hnew.SetDirectory(0);
           return hnew;
        """	
        MM_product_other = self.histos["effMM_otherQ2"] * self.histos["Yields_CorrMM"]
        MM_product_jps   = self.histos["effMM_jps"] * self.histos["Yields_CorrMM"]

        EE_product_other = self.histos["effEE_otherQ2"] * self.histos["Yields_CorrEE"]
        EE_product_jps   = self.histos["effEE_jps"] * self.histos["Yields_CorrEE"]
        try : 
            muon_fd = ( MM_product_other.Integral() / MM_product_jps.Integral()    )  * (  self.histos["effMM_jps"].Integral() / self.histos["effMM_otherQ2"].Integral() )
            ele_fd  = ( EE_product_other.Integral() / EE_product_jps.Integral()    )  * (  self.histos["effEE_jps"].Integral() / self.histos["effEE_otherQ2"].Integral() )
            self.flaPar = muon_fd/ele_fd -1.
        except : 
            self.flaPar = 0   
    def getFlaPar(self):
        return self.flaPar

import matplotlib.pyplot as plt
plt.ioff()
plt.rcdefaults()
import numpy as np
import matplotlib.pyplot as plt
params = {'legend.fontsize': 50,
          'legend.handlelength': 40,
           'font.weight' : 'bold', 
            'font.size' : 20 }
plt.rcParams.update(params)
plt.rcdefaults()
_VARLISTS = { "RK" :  [ ("B_PT" ,     r'$p_{T}(B)$'         ),
                          ("B_P" ,      r'$p(B)$'         ),
                          ("B_ETA" ,    r'$\eta(B)$'           ),
                          ("B_FDCHI2" , r'$FD\chi^{2}(B)$'             ),
                          ("B_IPCHI2" , r'$IP\chi^{2}(B)$'             ),
                          ("B_DIRA_OWNPV" ,  r'$DIRA_{OWNPV}(B)$'            ),
                          ("B_VTXCHI2DOF" ,  r'$\chi^{2}_{VTX}(B)$'             ),
                          ("B_ThetaL_custom" ,  r'$\theta_{\ell\ell}^{cm}$'             ),
#                           ("B_Phi_custom" ,     r'$\phi_{cm}$'         ),
#                           ("B_ThetaK_custom" ,  r'$\theta_{K}$'            ),
                          ("B_TAU" ,r'$\tau(B)$'),
                          ("cat_wMVA_lowcen" ,   r'$w_{MVA}^{lowcen}$'           ),
                          ("nVeloTracks" ,r'$n_{VeloTracks}$'),
                          ("nTracks" ,r'$n_{Tracks}$'),
                          ("nSPDHits" ,r'$n_{SPDHits}$'),
                          ("JPs_PT" ,r'$p_{T}(J/\psi)$'),
                          ("JPs_P" ,r'$p(J/\psi)$'),
                          ("JPs_ETA" ,r'$\eta(J/\psi)$'),
                          ("JPs_DIRA_OWNPV" ,r'$DIRA_{OWNPV}(J/\psi)$'),
                          ("JPs_ENDVERTEX_CHI2" ,r'$VTX\chi^{2}(J/\psi)$'),
                          ("JPs_FDCHI2_OWNPV" ,r'$FD_{OWNPV}\chi^{2}(J/\psi)$'),
                          ("JPs_IPCHI2_OWNPV" ,r'$IP_{OWNPV}\chi^{2}(J/\psi)$'),
                          ("L1L2_OA" ,r'$\theta_{lab}(\ell\ell)$'),
                          ("MinPt_L" ,r'$min(p_{T})\ell\ell$'),
                          ("MaxPt_L" ,r'$max(p_{T})\ell\ell$'),
                          ("MinEta_L" ,r'$min(\eta)\ell\ell$'),
                          ("MaxEta_L" ,r'$max(\eta)\ell\ell$'),
                          ("MaxIPCHI2_OWNPV_L" , r'$max(IP\chi^{2})\ell\ell$'),
                          ("MinIPCHI2_OWNPV_L" ,r'$min(IP\chi^{2})\ell\ell$'),
                          ("Pt_K" ,r'$p_{T}(K)$'),
                          ("Eta_K" ,r'$\eta(K)$'),
                          ("IPCHI2_OWNPV_K" ,r'$IP\chi^{2}_{OWNPV}(K)$')] ,             
              "RKst" :  [ ("B_PT" ,     r'$p_{T}(B)$'         ),
                          ("B_P" ,      r'$p(B)$'         ),
                          ("B_ETA" ,    r'$\eta(B)$'           ),
                          ("B_FDCHI2" , r'$FD\chi^{2}(B)$'             ),
                          ("B_IPCHI2" , r'$IP\chi^{2}(B)$'             ),
                          ("B_DIRA_OWNPV" ,  r'$DIRA_{OWNPV}(B)$'            ),
                          ("B_VTXCHI2DOF" ,  r'$\chi^{2}_{VTX}(B)$'             ),
                          ("B_ThetaL_custom" ,  r'$\theta_{\ell\ell}^{cm}$'             ),
                          ("B_Phi_custom" ,     r'$\phi_{cm}$'         ),
                          ("B_ThetaK_custom" ,  r'$\theta_{K}$'            ),
                          ("B_TAU" ,r'$\tau(B)$'),
                          ("cat_wMVA_lowcen" ,   r'$w_{MVA}^{lowcen}$'           ),
                          ("nVeloTracks" ,r'$n_{VeloTracks}$'),
                          ("nTracks" ,r'$n_{Tracks}$'),
                          ("nSPDHits" ,r'$n_{SPDHits}$'),
                          ("JPs_PT" ,r'$p_{T}(J/\psi)$'),
                          ("JPs_P" ,r'$p(J/\psi)$'),
                          ("JPs_ETA" ,r'$\eta(J/\psi)$'),
                          ("JPs_DIRA_OWNPV" ,r'$DIRA_{OWNPV}(J/\psi)$'),
                          ("JPs_ENDVERTEX_CHI2" ,r'$VTX\chi^{2}(J/\psi)$'),
                          ("JPs_FDCHI2_OWNPV" ,r'$FD_{OWNPV}\chi^{2}(J/\psi)$'),
                          ("JPs_IPCHI2_OWNPV" ,r'$IP_{OWNPV}\chi^{2}(J/\psi)$'),
                          ("L1L2_OA" ,r'$\theta_{lab}(\ell\ell)$'),
                          ("MinPt_L" ,r'$min(p_{T})\ell\ell$'),
                          ("MaxPt_L" ,r'$max(p_{T})\ell\ell$'),
                          ("MinEta_L" ,r'$min(\eta)\ell\ell$'),
                          ("MaxEta_L" ,r'$max(\eta)\ell\ell$'),
                          ("MaxIPCHI2_OWNPV_L" , r'$max(IP\chi^{2})\ell\ell$'),
                          ("MinIPCHI2_OWNPV_L" ,r'$min(IP\chi^{2})\ell\ell$'),
                          ("Kst_ENDVERTEX_CHI2" ,r'$VTX\chi^{2}(K^{\ast0})$'),
                          ("Kst_IPCHI2_OWNPV" ,r'$IP_{OWNPV}\chi^{2}(K^{\ast0})$'),
                          ("Kst_FDCHI2_OWNPV" ,r'$FD_{OWNPV}\chi^{2}(K^{\ast0})$'),
                          ("Kst_DIRA_OWNPV" ,r'$DIRA_{OWNPV}(K^{\ast0})$'),
                          ("Kst_PT" ,r'$p_{T}(K^{\ast0})$'),
                          ("Kst_ETA" ,r'$\eta(K^{\ast0})$'),
                          ("MinPt_H" ,r'$min(p_{T})(K\pi)$'),
                          ("MaxPt_H" ,r'$max(p_{T})(K\pi)$'),
                          ("MinEta_H" ,r'$min(\eta)(K\pi)$'),
                          ("MaxEta_H" ,r'$max(\eta)(K\pi)$'),
                          ("MaxIPCHI2_OWNPV_H" ,r'$max(IP\chi^{2})(K\pi)$'),
                          ("Pt_K" ,r'$p_{T}(K)$'),
                          ("Eta_K" ,r'$\eta(K)$'),
                          ("IPCHI2_OWNPV_K" ,r'$IP\chi^{2}_{OWNPV}(K)$'),
                          ("Pt_Pi" ,r'$p_{T}(\pi)$'),
                          ("Eta_Pi" ,r'$\eta(\pi)$'),
                          ("IPCHI2_OWNPV_Pi" ,r'$IP\chi^{2}_{OWNPV}(\pi)$')]
}
variables = _VARLISTS['RKst']
y_values = np.arange(0, len(variables))
print(len(y_values))
print(len(variables))
# """ RKST low q2 """
# for sysQ2 in ["psi","low","central"] : #,"psi"]: 

for sysQ2 in ["low","central", "psi"] : #,"psi"]: 
    variables = _VARLISTS["RKst"]# ["B_PT", "B_P", "B_ETA", "nTracks", "nSPDHits", "B_FDCHI2", "B_IPCHI2", "B_DIRA_OWNPV","B_VTXCHI2DOF"]
    #     inFile = "results_flatness/rJPsi_Plots_RKst_AllCorrections_DTF_AnaNote_WithBS_RECOONLY_flatness_FIT_Flatness_October2020_KSTJPS_DTF_NOBREMSPLIT_Syst_{}.root".format(sysQ2)
    #rJPsi_Plots_RKst_AllCorrections_AllSteps_ANANOTE_flatness_FIT_Flatness_October2020_KSTJPS_DTF_NOBREMSPLIT_Syst_psi.root    
    inFile = "results_flatness/rJPsi_Plots_RKst_AllCorrections_AllSteps_ANANOTE_flatness_FIT_Flatness_October2020_KSTJPS_DTF_NOBREMSPLIT_Syst_{}.root".format(sysQ2)    
    wBp_results = {} 
    wB0_results = {} 
    wno_results = {}     
    for year in ["R1","R2p1","R2p2"]:    
        for trg in ["L0I", "L0L"] :           
            wBp_results.update( { '{}-{}'.format( year,trg) : [] } )
            wB0_results.update( { '{}-{}'.format( year,trg) : [] } )        
            wno_results.update( { '{}-{}'.format( year,trg) : [] } )
            for var in variables :          
                allCorrections_Bp =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "Bp_fit", year = year, trg = trg, trgConf = "exclusive", wOpt = "PID_TRK_BDT_BKIN_MULT_RECO_L0_HLT_nTracks", sysQ2 = sysQ2)
                allCorrections_B0 =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "B0_fit", year = year, trg = trg, trgConf = "exclusive", wOpt = "PID_TRK_BDT_BKIN_MULT_RECO_L0_HLT_nTracks", sysQ2 = sysQ2)
                allCorrections_no =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "Bp_fit", year = year, trg = trg, trgConf = "exclusive", wOpt = "no", sysQ2 = sysQ2)
                
                var_res_wBp_chain = 100*allCorrections_Bp.getFlaPar()
                var_res_wB0_chain = 100*allCorrections_B0.getFlaPar()
                var_res_no        = 100*allCorrections_no.getFlaPar()
                
                wB0_results['{}-{}'.format( year,trg)].append(var_res_wB0_chain )
                wBp_results['{}-{}'.format( year,trg)].append(var_res_wBp_chain )
                wno_results['{}-{}'.format( year,trg)].append(var_res_no )
            fig, ax = plt.subplots(figsize=(60,80), tight_layout = True)            
            varLabels = [v[0] for v in variables]
            varAxis   = [v[1] for v in variables]
            LABELS = np.array( varAxis)            
            print( len(varAxis))
            print( len(varLabels))
            y_pos     = np.arange(len(LABELS))
            y_pos_Bp  = y_pos+0.15
            y_pos_B0  = y_pos-0.15
            


            performance_B0 = np.array( wB0_results['{}-{}'.format( year,trg)] )
            performance_Bp = np.array( wBp_results['{}-{}'.format( year,trg)] )
            performance_no = np.array( wno_results['{}-{}'.format( year,trg)] )

            ax.barh(y_pos_B0, performance_B0, align='center',
                    color='green', ecolor='black', height = 0.1 , label = r'$w(B^{0})$')
            ax.barh(y_pos_Bp, performance_Bp, align='center',
                    color='blue', ecolor='black', height = 0.1 , label = r'$w(B^{+})$')
            
            ax.barh(y_pos, performance_no, align='center',
                    color='black', ecolor='black', height = 0.1 , label = r'$w_{no}$')            
            ax.set_yticks(y_pos)
            ax.set_yticklabels(LABELS, fontsize = 50,  fontweight="bold")
            # ax.invert_yaxis()  # labels read top-to-bottom
            ax.set_xlabel(r'$d_{f}$ [%]', fontsize = 40,  fontweight="bold")
            def title( year, trigger, project, q2bin) : 
                titles = { 
                    'R1-L0I-RKst-low'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R1-L0L-RKst-low'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p1-L0I-RKst-low'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p1-L0L-RKst-low'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p2-L0I-RKst-low'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p2-L0L-RKst-low'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',                    
                    'R1-L0I-RKst-central'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R1-L0L-RKst-central'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p1-L0I-RKst-central'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p1-L0L-RKst-central'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p2-L0I-RKst-central'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p2-L0L-RKst-central'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',                                          
                    'R1-L0I-RKst-psi'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R1-L0L-RKst-psi'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p1-L0I-RKst-psi'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p1-L0L-RKst-psi'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p2-L0I-RKst-psi'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p2-L0L-RKst-psi'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',             
                    
                    'R1-L0I-RK-low'  : r'(11+12) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R1-L0L-RK-low'  : r'(11+12) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p1-L0I-RK-low'  : r'(15+16) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p1-L0L-RK-low'  : r'15+16) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p2-L0I-RK-low'  : r'(17+18) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p2-L0L-RK-low'  : r'(17+18) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',                    
                    'R1-L0I-RK-central'  : r'(11+12) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R1-L0L-RK-central'  : r'(11+12) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p1-L0I-RK-central'  : r'(15+16) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p1-L0L-RK-central'  : r'15+16) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p2-L0I-RK-central'  : r'(17+18) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p2-L0L-RK-central'  : r'(17+18) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',                                          
                    'R1-L0I-RK-psi'  : r'(11+12) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R1-L0L-RK-psi'  : r'(11+12) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p1-L0I-RK-psi'  : r'(15+16) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p1-L0L-RK-psi'  : r'15+16) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p2-L0I-RK-psi'  : r'(17+18) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p2-L0L-RK-psi'  : r'(17+18) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',                              
                }
                
                return titles['-'.join([year, trg, project, q2bin])]
            ax.set_title( title( year, trg, 'RKst', sysQ2),fontsize = 50,  fontweight="bold")
            ax.grid()
            ax.legend( fontsize = 50)
            ax.tick_params(axis='both', which='major', labelsize=50)
            ax.set_xlim(-5,5)
            # plt.annotate()
            print("Saving {}".format('RKst_{}_{}_{}_systFlatness.pdf'.format(sysQ2, year, trg)))
            fig.savefig('RKst_{}_{}_{}_systFlatness.pdf'.format(sysQ2, year, trg))                
variables = _VARLISTS['RK']
y_values = np.arange(0, len(variables))
print(len(y_values))
print(len(variables))
""" RK low q2 """
for sysQ2 in ["psi","low","central"] : #,"psi"]: 
    variables = _VARLISTS["RK"]# ["B_PT", "B_P", "B_ETA", "nTracks", "nSPDHits", "B_FDCHI2", "B_IPCHI2", "B_DIRA_OWNPV","B_VTXCHI2DOF"]
#     inFile = "results_flatness/rJPsi_Plots_RK_AllCorrections_DTF_AnaNote_WithBS_RECOONLY_flatness_FIT_Flatness_October2020_KJPS_DTF_NOBREMSPLIT_Syst_{}.root".format(sysQ2)
    inFile = "results_flatness/rJPsi_Plots_RK_AllCorrections_AllSteps_ANANOTE_flatness_FIT_Flatness_October2020_KJPS_DTF_NOBREMSPLIT_Syst_{}.root".format(sysQ2)    
    #inFile = "results_flatness/rJPsi_Plots_RKst_AllCorrections_DTF_AnaNote_WithBS_RECOONLY_flatness_FIT_Flatness_October2020_KSTJPS_DTF_NOBREMSPLIT_Syst_{}.root".format(sysQ2)    
    wBp_results = {} 
    wB0_results = {} 
    wno_results = {}     
    for year in ["R1","R2p1","R2p2"]:   
        if year == 'R2p1' : 
            yPatch = '16'
        else : 
            yPath = year
        for trg in ["L0I", "L0L"] :           
            wBp_results.update( { '{}-{}'.format( year,trg) : [] } )
            wB0_results.update( { '{}-{}'.format( year,trg) : [] } )        
            wno_results.update( { '{}-{}'.format( year,trg) : [] } )  
            for var in variables :          
                allCorrections_Bp =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "Bp_fit", year = yPath, trg = trg, trgConf = "exclusive", wOpt = "PID_TRK_BDT_BKIN_MULT_RECO_L0_HLT_nTracks", sysQ2 = sysQ2)
                
                allCorrections_B0 =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "B0_fit", year = yPath, trg = trg, trgConf = "exclusive", wOpt = "PID_TRK_BDT_BKIN_MULT_RECO_L0_HLT_nTracks", sysQ2 = sysQ2)
                allCorrections_no =  FlatnessParEvaluator( inFile = inFile, varID = var[0], wConfig = "B0_fit", year = yPath, trg = trg, trgConf = "exclusive", wOpt = "no", sysQ2 = sysQ2)
                
                var_res_wBp_chain = 100*allCorrections_Bp.getFlaPar()
                var_res_wB0_chain = 100*allCorrections_B0.getFlaPar()
                var_res_wno_chain = 100*allCorrections_no.getFlaPar()
                
                wB0_results['{}-{}'.format( year,trg)].append(var_res_wB0_chain )
                wBp_results['{}-{}'.format( year,trg)].append(var_res_wBp_chain )
                wno_results['{}-{}'.format( year,trg)].append(var_res_wno_chain )
                
            fig, ax = plt.subplots(figsize=(60,80), tight_layout = True)            
            varLabels = [v[0] for v in variables]
            varAxis   = [v[1] for v in variables]
            LABELS = np.array( varAxis)            
            print( len(varAxis))
            print( len(varLabels))
            y_pos     = np.arange(len(LABELS))
            y_pos_Bp  = y_pos+0.15
            y_pos_B0  = y_pos-0.15



            performance_B0 = np.array( wB0_results['{}-{}'.format( year,trg)] )
            performance_Bp = np.array( wBp_results['{}-{}'.format( year,trg)] )
            performance_no = np.array( wno_results['{}-{}'.format( year,trg)] )

            ax.barh(y_pos_B0, performance_B0, align='center',
                    color='green', ecolor='black', height = 0.1 , label = r'$w(B^{0})$')
            ax.barh(y_pos_Bp, performance_Bp, align='center',
                    color='blue', ecolor='black', height = 0.1 , label = r'$w(B^{+})$')
            ax.barh(y_pos, performance_no, align='center',
                    color='black', ecolor='black', height = 0.1 , label = r'$w_{no}$')            
            ax.set_yticks(y_pos)
            ax.set_yticklabels(LABELS, fontsize = 50,  fontweight="bold")
            # ax.invert_yaxis()  # labels read top-to-bottom
            ax.set_xlabel(r'$d_{f}$ [%]', fontsize = 40,  fontweight="bold")
            def title( year, trigger, project, q2bin) : 
                titles = { 
                    'R1-L0I-RKst-low'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R1-L0L-RKst-low'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p1-L0I-RKst-low'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p1-L0L-RKst-low'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p2-L0I-RKst-low'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p2-L0L-RKst-low'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{low}$ [L0L secondary]',                    
                    'R1-L0I-RKst-central'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R1-L0L-RKst-central'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p1-L0I-RKst-central'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p1-L0L-RKst-central'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p2-L0I-RKst-central'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p2-L0L-RKst-central'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{central}$ [L0L secondary]',                                          
                    'R1-L0I-RKst-psi'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R1-L0L-RKst-psi'  : r'(11+12) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p1-L0I-RKst-psi'  : r'(15+16) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p1-L0L-RKst-psi'  : r'15+16) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p2-L0I-RKst-psi'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p2-L0L-RKst-psi'  : r'(17+18) $R_{K^{\ast0}}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',             
                    
                    'R1-L0I-RK-low'  : r'(11+12) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R1-L0L-RK-low'  : r'(11+12) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p1-L0I-RK-low'  : r'(15+16) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p1-L0L-RK-low'  : r'15+16) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',
                    'R2p2-L0I-RK-low'  : r'(17+18) $R_{K}$ $q^{2}_{low}$ [L0I primary]',
                    'R2p2-L0L-RK-low'  : r'(17+18) $R_{K}$ $q^{2}_{low}$ [L0L secondary]',                    
                    'R1-L0I-RK-central'  : r'(11+12) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R1-L0L-RK-central'  : r'(11+12) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p1-L0I-RK-central'  : r'(15+16) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p1-L0L-RK-central'  : r'15+16) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',
                    'R2p2-L0I-RK-central'  : r'(17+18) $R_{K}$ $q^{2}_{central}$ [L0I primary]',
                    'R2p2-L0L-RK-central'  : r'(17+18) $R_{K}$ $q^{2}_{central}$ [L0L secondary]',                                          
                    'R1-L0I-RK-psi'  : r'(11+12) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R1-L0L-RK-psi'  : r'(11+12) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p1-L0I-RK-psi'  : r'(15+16) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p1-L0L-RK-psi'  : r'15+16) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',
                    'R2p2-L0I-RK-psi'  : r'(17+18) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0I primary]',
                    'R2p2-L0L-RK-psi'  : r'(17+18) $R_{K}$ $q^{2}_{\psi(2S)}$ [L0L secondary]',                              
                }
                
                return titles['-'.join([year, trg, project, q2bin])]
            ax.set_title( title( year, trg, 'RK', sysQ2),fontsize = 50,  fontweight="bold")
            ax.grid()
            ax.legend( fontsize = 50)
            ax.tick_params(axis='both', which='major', labelsize=50)
            ax.set_xlim(-5,5)
            # plt.annotate()
            print("Saving {}".format('RK_{}_{}_{}_systFlatness.pdf'.format(sysQ2, year, trg)))
            fig.savefig('RK_{}_{}_{}_systFlatness.pdf'.format(sysQ2, year, trg))     